@@ -100,7 +100,7 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
       !strcasecmp(src_charset, "CP932") || !strcasecmp(src_charset, "WINDOWS-932") ||
       !strcasecmp(src_charset, "MS932") || !strncasecmp(src_charset, "X-SJIS", 6)) {
     src_ch = STR_SJIS;
-  } else if (!strncasecmp(src_charset, "ISO-2022-JP", 11)) {
+  } else if (!strncasecmp(src_charset, "ISO-2022-JP", 11) || !strncasecmp(src_charset, "X-WINDOWS-ISO2022JP", 19)) {
     src_ch = STR_JIS;
   } else if (!strncasecmp(src_charset, "UTF-8", 5) || !strncasecmp(src_charset, "UTF8", 4)) {
     src_ch = STR_UTF8;
@@ -160,7 +160,7 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
       !strcasecmp(dest_charset, "CP932") || !strcasecmp(dest_charset, "WINDOWS-932") ||
       !strcasecmp(dest_charset, "MS932") || !strncasecmp(dest_charset, "X-SJIS", 6)) {
     dest_ch = STR_SJIS;
-  } else if (!strncasecmp(dest_charset, "ISO-2022-JP", 11)) {
+  } else if (!strncasecmp(dest_charset, "ISO-2022-JP", 11) || !strncasecmp(dest_charset, "X-WINDOWS-ISO2022JP", 19)) {
     dest_ch = STR_JIS;
   } else if (!strncasecmp(dest_charset, "UTF-8", 5) || !strncasecmp(dest_charset, "UTF8", 4)) {
     dest_ch = STR_UTF8;
@@ -577,8 +577,11 @@ const char* jis_auto_detect(const char *str) {
 
     // Judgment of CP932 and ISO-2022-JP
     for (i = 0; i < length && i < max; i++) {
-      if (issjis1((unsigned char)str[i]) && (i + 1 < length)) {
-        if (issjis2((unsigned char)str[i + 1])) {
+      unsigned char c, d;
+      c = (unsigned char)str[i];
+      if (issjis1(c) && (i + 1 < length)) {
+        d = (unsigned char)str[i+1];
+        if (issjis2(d)) {
           badJIS += 10;
           i++;
         } else if (0x989F <= (((unsigned int)str[i-1] << 8) | (unsigned int)str[i])) {
@@ -1178,7 +1181,9 @@ char* sjis_to_eucjp(const char *str) {
     i = 0;
     j = 0;
     while (i < length) {
-      if (issjis1(c = str[i]) && issjis2(d = str[i+1])) {
+      c = str[i];
+      d = c?(str[i+1]):0;
+      if (issjis1(c) && issjis2(d)) {
         if (c <= 0x9F) {
           if (d < 0x9F) {
             c = (c << 1) - 0xE1;
@@ -1244,6 +1249,8 @@ char* jis_to_eucjp(const char *str) {
     i = 0;
     j = 0;
     while (i < length) {
+      c = str[i];
+      d = c?(str[i+1]):0;
       if (str[i] == 0x1B) {
         if (str[i+1] == '$') {
           if (i+2 < length && (str[i+2] == '@' || str[i+2] == 'B')) {
@@ -1277,7 +1284,7 @@ char* jis_to_eucjp(const char *str) {
       } else if ((mode == HANKANA) && (0x20 <= str[i] && str[i] <= 0x5F)) {
         buf[j++] = 0x8E;
         buf[j++] = (str[i] | 0x80);
-      } else if (issjis1(c = str[i]) && issjis2(d = str[i+1])) {
+      } else if (issjis1(c) && issjis2(d)) {
         if (c <= 0x9F) {
           if (d < 0x9F) {
             c = (c << 1) - 0xE1;
@@ -1524,7 +1531,7 @@ char* sjis_to_utf8(const char *str) {
     j = 0;
     while (i < length) {
       c = str[i];
-      d = str[i+1];
+      d = c?(str[i+1]):0;
       if ((issjis1(c) && issjis2(d)) || ishankana(c)) {
         int pos, old_pos, min, max, s;
 
@@ -1539,7 +1546,7 @@ char* sjis_to_utf8(const char *str) {
               break;
             }
           }
-          if (UNICODE_EUCJP_TABLE_MAX-1 < max) {
+          if (SJIS_UNICODE_TABLE_MAX-1 < max) {
             max = SJIS_UNICODE_TABLE_MAX-1;
           }
         } else {
@@ -1752,7 +1759,7 @@ char *jis_to_sjis(const char *str) {
   if (str != NULL) {
     // Memory allocation
     length = strlen(str);
-    buf = new char[length + 2048];
+    buf = new char[length * 3 + 2048];
 
     // ISO-2022-JP to CP932
     shifted = IS_ROMAN;
@@ -1792,15 +1799,15 @@ char *jis_to_sjis(const char *str) {
       if (normal_flg) {
         switch (shifted) {
           case IS_KANJI:
-            hi = (int) from[i] & 0xff;
-            lo = (int) from[i+1] & 0xff;
+            hi = (int) from[i] & 0xFF;
+            lo = (int) from[i+1] & 0xFF;
             if (hi & 1) {
-              c = ((hi / 2 + (hi < 0x5f ? 0x71 : 0xb1)) << 8) |
-                  (lo + (lo >= 0x60 ? 0x20 : 0x1f));
+              c = ((hi / 2 + (hi < 0x5F ? 0x71 : 0xB1)) << 8) |
+                  (lo + (lo >= 0x60 ? 0x20 : 0x1F));
             } else {
-              c = ((hi / 2 + (hi < 0x5f ? 0x70 : 0xb0)) << 8) | (lo + 0x7e);
+              c = ((hi / 2 + (hi < 0x5F ? 0x70 : 0xB0)) << 8) | (lo + 0x7E);
             }
-            buf[j++] = (c >> 8) & 0xff;
+            buf[j++] = (c >> 8) & 0xFF;
             buf[j++] = c;
             i += 2;
             break;
